import pytest
import os

from ..cli.lib.mql_api import MQLClient, BackendClient
from ..cli.lib.shared import MdoModelKey, Constraint


@pytest.fixture
def mql_client() -> MQLClient:  # noqa: D
    api_key = os.getenv("TRANSFORM_API_KEY")
    url = "https://mql-query.staging.transformdata.io"

    # This is quite silly indeed
    assert api_key is not None

    return MQLClient(api_key, url)


@pytest.fixture
def backend_client() -> BackendClient:  # noqa: D
    api_key = os.getenv("TRANSFORM_API_KEY")

    # This is quite silly indeed
    assert api_key is not None

    return BackendClient(api_key)


@pytest.mark.skip(reason="skip until next MQL Server deploy")
def test_materialize(mql_client: MQLClient) -> None:  # noqa: D
    model_key = MdoModelKey("1", "transform-test", "master", "7ea4393563ad5abcc626dd03926816dfece4c52a")
    constraint = Constraint("SET", "country", ["US", "CA"])

    result = mql_client.materialize(
        model_key=model_key,
        table="public.myTable",
        metrics=["bookings"],
        dimensions=["country", "ds"],
        constraint=constraint,
        cache_mode="rw",
    )

    assert result is not None


@pytest.mark.skip(reason="skip until next MQL Server deploy")
def test_create_query(mql_client: MQLClient) -> None:  # noqa: D
    model_key = MdoModelKey("1", "transform-test", "master", "7ea4393563ad5abcc626dd03926816dfece4c52a")
    constraint = Constraint("SET", "country", ["US", "CA"])

    result = mql_client.create_query(
        model_key=model_key,
        metrics=["bookings"],
        dimensions=["country", "ds"],
        constraint=constraint,
        order=["-ds"],
        limit=1,
        cache_mode="rw",
    )
    assert result is not None


def test_get_health(mql_client: MQLClient) -> None:  # noqa: D
    result = mql_client.get_health()
    assert result.text == "Healthy."


def test_get_latest_model_key(backend_client: BackendClient) -> None:  # noqa: D
    result = backend_client.get_latest_model_key()
    assert result is not None


def test_get_me(backend_client: BackendClient) -> None:  # noqa: D
    result = backend_client.get_me()
    assert result is not None


def test_get_my_org(backend_client: BackendClient) -> None:  # noqa: D
    result = backend_client.get_my_org()
    assert result is not None


@pytest.mark.skip(reason="skip until next MQL Server deploy")
def test_get_query_status(mql_client: MQLClient) -> None:  # noqa: D
    model_key = MdoModelKey("1", "transform-test", "master", "7ea4393563ad5abcc626dd03926816dfece4c52a")
    constraint = Constraint("SET", "country", ["US", "CA"])

    query_id = mql_client.create_query(
        model_key=model_key,
        metrics=["bookings"],
        dimensions=["country", "ds"],
        constraint=constraint,
    )

    result = mql_client.get_query_status(query_id)
    assert result is not None


@pytest.mark.skip(reason="skip until next MQL Server deploy")
def test_get_query_page_as_df(mql_client: MQLClient) -> None:  # noqa: D
    model_key = MdoModelKey("1", "transform-test", "master", "7ea4393563ad5abcc626dd03926816dfece4c52a")
    constraint = Constraint("SET", "country", ["US", "CA"])

    query_id = mql_client.create_query(
        model_key=model_key,
        metrics=["bookings"],
        dimensions=["country", "ds"],
        constraint=constraint,
    )

    result = mql_client.get_query_page_as_df(query_id, None)
    assert result.cursor is not None
    assert result.df is not None
